rem Expression Filters

BEGIN
  DBMS_EXPFIL.UNASSIGN_ATTRIBUTE_SET
  ( expr_tab => 'Consumer'
  , expr_col => 'Interest'
  );
  DBMS_EXPFIL.DROP_ATTRIBUTE_SET
  ( attr_set => 'Car4Sale'
  );
END;
/
drop table consumer
/
drop TYPE Car4Sale 
/
drop table inventory
/

set serveroutput on 99999
set feedback on
set termout on
set echo on
set pagesize 1000
set linesize 200
clear screen



-- Use type definition as the source of the attribute set for an expression
CREATE OR REPLACE TYPE Car4Sale AS OBJECT
(Model VARCHAR2(20),
 Year NUMBER,
 Price NUMBER,
 Mileage NUMBER);
/
pause

-- Defining an attribute set
BEGIN
  DBMS_EXPFIL.CREATE_ATTRIBUTE_SET
  ( attr_set => 'Car4Sale' -- base attribute set on the Car4Sale object type
  , from_type => 'YES');
END;
/

-- Create user table with expression column (or add new varchar2 column to existing table) 
CREATE TABLE Consumer 
( CId NUMBER,
 Zipcode NUMBER,
 Phone VARCHAR2(12),
 Interest VARCHAR2(200)
);


--Assign attribute set to the expression column in the table
BEGIN
  DBMS_EXPFIL.ASSIGN_ATTRIBUTE_SET 
  ( attr_set => 'Car4Sale'
  , expr_tab => 'Consumer'
  , expr_col => 'Interest'
  );
END;
/


-- Insert expressions in the table
INSERT INTO Consumer VALUES (1, 32611, '917 768 4633','Model=''Taurus'' and Price < 15000 and Mileage < 25000');
INSERT INTO Consumer VALUES (2, 03060, '603 983 3464','Model=''Mustang'' and Year > 1999 and Price < 20000');


-- Apply the SQL EVALUATE operator

rem See if we can find anyone interested in a 2000 Mustang, priced at 18000

SELECT * 
FROM   Consumer 
WHERE  EVALUATE 
       ( Consumer.Interest
       , Car4Sale
         ( 'Mustang'
         , 2000
         , 18000
         , 22000
         ).getVarchar()     -- MEMBER getVarchar() API --
       ) = 1

pause

/

pause
rem build inventory table

CREATE TABLE Inventory 
( Model VARCHAR2(20)
, Year NUMBER
, Price NUMBER
, Mileage NUMBER
);

insert into inventory (model, year, price, mileage) values ('BMW 305',2001,9100,1500);
insert into inventory (model, year, price, mileage) values ('Taurus',1984,129100,500);
insert into inventory (model, year, price, mileage) values ('Taunus',1997,19107,2500);
insert into inventory (model, year, price, mileage) values ('Volkswagen Golf',2004,1100,6500);
insert into inventory (model, year, price, mileage) values ('Mustang',1997,19107,2500);

INSERT INTO Consumer VALUES (3, 0540, '+31621294823','Year > 1990 and mileage < 100000');
INSERT INTO Consumer VALUES (4, 0210, '+3179323026','not regexp_like(Model, ''(bmw)'',''i'') and Year > 1980 and price < 10000');


rem Finding a match between the two tables:

SELECT c.cid
,      c.phone
,      i.model
,      i.year
,      i.price
,      i.mileage
FROM   Consumer  c
,      Inventory i
WHERE  EVALUATE 
       ( c.Interest
       , Car4Sale
         ( i.model
         , i.year
         , i.price
         , i.mileage         
         ).getVarchar()     -- MEMBER getVarchar() API --
       ) = 1

pause
/

pause

rem Reporting on Interested Customers for New Inventory:

CREATE or replace 
TRIGGER activechk 
AFTER insert OR update 
ON Inventory
FOR EACH ROW
  DECLARE
	cursor c1 (ditem VARCHAR2) 
	is
	  SELECT c.cid
	  ,      c.phone 
	  FROM   Consumer c
	  WHERE  EVALUATE (c.Interest, ditem) = 1
	  ;
    ditem VARCHAR2(200);
BEGIN
  ditem := Car4Sale( :new.Model, :new.Year
                   , :new.Price, :new.Mileage
                   ).getVarchar(); 
  for cur in c1(ditem) loop  
    DBMS_OUTPUT.PUT_LINE(' For New or Updated Car '||:new.Model||' ('||:new.year||') '||' Call customer '||cur.cId||'@'||cur.phone);  
  end loop;
END;
/

insert into inventory (model, year, price, mileage) values ('Toyota',1990,1500,79107)


pause
/
pause

insert into inventory (model, year, price, mileage) values ('Jaguar',1975,500,279107)
/
pause

update inventory
set    year = 1981
where  model ='Jaguar'
/
pause

rollback
/

