rem Expression Filters

BEGIN
  DBMS_EXPFIL.UNASSIGN_ATTRIBUTE_SET
  ( expr_tab => 'Business_Rules'
  , expr_col => 'br_expression'
  );
  DBMS_EXPFIL.DROP_ATTRIBUTE_SET
  ( attr_set => 'BusinessRuleConditions'
  );
END;
/

drop table business_rules
/

drop type string_tbl_type 
/

drop TYPE BusinessRuleConditions
/

set serveroutput on 99999
set feedback on
set termout on
set echo on
set pagesize 1000
set linesize 200
clear screen


create table business_rules
( br_label      varchar2(25)
, br_msg        varchar2(20)
, br_tbl        varchar2(32)
, br_expression varchar2(2000)
)
/


CREATE OR REPLACE TYPE BusinessRuleConditions AS OBJECT
( Condition1 VARCHAR2(20)
, Condition2 VARCHAR2(20)
, Condition3 VARCHAR2(20)
, Condition4 VARCHAR2(20)
, Condition5 VARCHAR2(20)
);
/

BEGIN
  DBMS_EXPFIL.CREATE_ATTRIBUTE_SET
  ( attr_set => 'BusinessRuleConditions'
  , from_type => 'YES'
  );
END;
/

BEGIN
  DBMS_EXPFIL.ASSIGN_ATTRIBUTE_SET 
  ( attr_set => 'BusinessRuleConditions'
  , expr_tab => 'Business_Rules'
  , expr_col => 'br_expression'
  );
END;
/

pause

update emp set comm = 150 where job ='ANALYST'
/

clear screen

rem create some business rules on EMP and DEPT:



INSERT INTO Business_rules VALUES ('EMP1', '001','EMP', 'Condition1 =''SALESMAN'' or Condition2 is null');
INSERT INTO Business_rules VALUES ('EMP2', '002','EMP', '(sysdate - to_date(condition3)) <8800');
INSERT INTO Business_rules VALUES ('DPT1', '001','DEPT', 'Condition2 <>''SALES'' or Condition3 <> ''PARIS'' ');
INSERT INTO Business_rules VALUES ('DPT1', '001','DEPT', 'Condition3 <> ''NEW YORK'' ');

create or replace 
type string_tbl_type as table of varchar2(50)
/

SELECT br.br_expression Business_Rule
,      ' violated by '||count(d.deptno)||' records' 
,      cast( collect(dname) as string_tbl_type ) Perpetrators
FROM   Business_Rules br
,      Dept d
WHERE  EVALUATE        
       ( br.br_expression
       , BusinessRuleConditions.getVarchar
         ( d.deptno
	 , d.dname
	 , d.loc
	 , null
	 , null
         )
       ) = 0
AND    br.br_tbl = 'DEPT'       
GROUP BY br.br_expression

pause
/

rem now look for violations in EMP
pause

SELECT br.br_expression Business_Rule
,      ' violated by '||count(e.empno)||' records' 
,      cast( collect(ename) as string_tbl_type ) Perpetrators
FROM   Business_Rules br
,      Emp e
WHERE  EVALUATE        
       ( br.br_expression
       , BusinessRuleConditions.getVarchar
         ( e.job
	 , e.comm
	 , e.hiredate
	 , e.deptno
	 , e.mgr
         )
       ) = 0
AND    br.br_tbl = 'EMP'       
GROUP BY br.br_expression
/

pause
rem add another rule: Managers do not need managers
INSERT INTO Business_rules VALUES ('EMP5', '021','EMP', 'Condition1 <>''MANAGER'' or Condition5 is null ');

SELECT br.br_expression Business_Rule
,      ' violated by '||count(e.empno)||' records' 
,      cast( collect(ename) as string_tbl_type ) Perpetrators
FROM   Business_Rules br
,      Emp e
WHERE  EVALUATE        
       ( br.br_expression
       , BusinessRuleConditions.getVarchar
         ( e.job
	 , e.comm
	 , e.hiredate
	 , e.deptno
	 , e.mgr
         )
       ) = 0
AND    br.br_tbl = 'EMP'       
GROUP BY br.br_expression

pause

/
pause

rollback
/
clear screen
